#include <windows.h>
#include <d3dx9.h>
#include <cstdio>
#include <limits>
#include <utility>
#include <cstdlib>
#include <algorithm>

#pragma comment(lib,"d3d9.lib")
#pragma comment(lib,"d3dx9.lib")
#pragma comment(lib,"dxguid.lib")
#pragma comment(lib,"winmm.lib")

using namespace std;

enum TIMER_COMMAND { TIMER_RESET, TIMER_START, TIMER_STOP, TIMER_ADVANCE,
TIMER_GETABSOLUTETIME, TIMER_GETAPPTIME, TIMER_GETELAPSEDTIME };

FLOAT __stdcall DXUtil_Timer( TIMER_COMMAND command )
{
	static BOOL     m_bTimerInitialized = FALSE;
	static BOOL     m_bUsingQPF         = FALSE;
	static BOOL     m_bTimerStopped     = TRUE;
	static LONGLONG m_llQPFTicksPerSec  = 0;

	// Initialize the timer
	if( FALSE == m_bTimerInitialized )
	{
		m_bTimerInitialized = TRUE;

		// Use QueryPerformanceFrequency() to get frequency of timer.  If QPF is
		// not supported, we will timeGetTime() which returns milliseconds.
		LARGE_INTEGER qwTicksPerSec;
		m_bUsingQPF = QueryPerformanceFrequency( &qwTicksPerSec );
		if( m_bUsingQPF )
			m_llQPFTicksPerSec = qwTicksPerSec.QuadPart;
	}

	if( m_bUsingQPF )
	{
		static LONGLONG m_llStopTime        = 0;
		static LONGLONG m_llLastElapsedTime = 0;
		static LONGLONG m_llBaseTime        = 0;
		double fTime;
		double fElapsedTime;
		LARGE_INTEGER qwTime;

		// Get either the current time or the stop time, depending
		// on whether we're stopped and what command was sent
		if( m_llStopTime != 0 && command != TIMER_START && command != TIMER_GETABSOLUTETIME)
			qwTime.QuadPart = m_llStopTime;
		else
			QueryPerformanceCounter( &qwTime );

		// Return the elapsed time
		if( command == TIMER_GETELAPSEDTIME )
		{
			fElapsedTime = (double) ( qwTime.QuadPart - m_llLastElapsedTime ) / (double) m_llQPFTicksPerSec;
			m_llLastElapsedTime = qwTime.QuadPart;
			return (FLOAT) fElapsedTime;
		}

		// Return the current time
		if( command == TIMER_GETAPPTIME )
		{
			double fAppTime = (double) ( qwTime.QuadPart - m_llBaseTime ) / (double) m_llQPFTicksPerSec;
			return (FLOAT) fAppTime;
		}

		// Reset the timer
		if( command == TIMER_RESET )
		{
			m_llBaseTime        = qwTime.QuadPart;
			m_llLastElapsedTime = qwTime.QuadPart;
			m_llStopTime        = 0;
			m_bTimerStopped     = FALSE;
			return 0.0f;
		}

		// Start the timer
		if( command == TIMER_START )
		{
			if( m_bTimerStopped )
				m_llBaseTime += qwTime.QuadPart - m_llStopTime;
			m_llStopTime = 0;
			m_llLastElapsedTime = qwTime.QuadPart;
			m_bTimerStopped = FALSE;
			return 0.0f;
		}

		// Stop the timer
		if( command == TIMER_STOP )
		{
			if( !m_bTimerStopped )
			{
				m_llStopTime = qwTime.QuadPart;
				m_llLastElapsedTime = qwTime.QuadPart;
				m_bTimerStopped = TRUE;
			}
			return 0.0f;
		}

		// Advance the timer by 1/10th second
		if( command == TIMER_ADVANCE )
		{
			m_llStopTime += m_llQPFTicksPerSec/10;
			return 0.0f;
		}

		if( command == TIMER_GETABSOLUTETIME )
		{
			fTime = qwTime.QuadPart / (double) m_llQPFTicksPerSec;
			return (FLOAT) fTime;
		}

		return -1.0f; // Invalid command specified
	}
	else
	{
		// Get the time using timeGetTime()
		static double m_fLastElapsedTime  = 0.0;
		static double m_fBaseTime         = 0.0;
		static double m_fStopTime         = 0.0;
		double fTime;
		double fElapsedTime;

		// Get either the current time or the stop time, depending
		// on whether we're stopped and what command was sent
		if( m_fStopTime != 0.0 && command != TIMER_START && command != TIMER_GETABSOLUTETIME)
			fTime = m_fStopTime;
		else
			fTime = timeGetTime() * 0.001;

		// Return the elapsed time
		if( command == TIMER_GETELAPSEDTIME )
		{   
			fElapsedTime = (double) (fTime - m_fLastElapsedTime);
			m_fLastElapsedTime = fTime;
			return (FLOAT) fElapsedTime;
		}

		// Return the current time
		if( command == TIMER_GETAPPTIME )
		{
			return (FLOAT) (fTime - m_fBaseTime);
		}

		// Reset the timer
		if( command == TIMER_RESET )
		{
			m_fBaseTime         = fTime;
			m_fLastElapsedTime  = fTime;
			m_fStopTime         = 0;
			m_bTimerStopped     = FALSE;
			return 0.0f;
		}

		// Start the timer
		if( command == TIMER_START )
		{
			if( m_bTimerStopped )
				m_fBaseTime += fTime - m_fStopTime;
			m_fStopTime = 0.0f;
			m_fLastElapsedTime  = fTime;
			m_bTimerStopped = FALSE;
			return 0.0f;
		}

		// Stop the timer
		if( command == TIMER_STOP )
		{
			if( !m_bTimerStopped )
			{
				m_fStopTime = fTime;
				m_fLastElapsedTime  = fTime;
				m_bTimerStopped = TRUE;
			}
			return 0.0f;
		}

		// Advance the timer by 1/10th second
		if( command == TIMER_ADVANCE )
		{
			m_fStopTime += 0.1f;
			return 0.0f;
		}

		if( command == TIMER_GETABSOLUTETIME )
		{
			return (FLOAT) fTime;
		}

		return -1.0f; // Invalid command specified
	}
}
using namespace std;

#define D3DFVF_MY_VERTEX	( D3DFVF_XYZ | D3DFVF_DIFFUSE )

//ԶmeshĶʽ
struct Vertex 
{
	float x, y, z; //
	DWORD diffuse; //ɫ
};


//OBBݽṹ
struct OBB
{
	D3DXVECTOR3 vCenter;		//obbе꣬ϵ
	D3DXVECTOR3 vVertex[8];		//obb8
	float		fDims[3];		//obbİ볤
};

//AABBݽṹ
struct AABB
{
	D3DXVECTOR3 vMin;	//AABB½꣬xyzΪСֵ
	D3DXVECTOR3 vMax;	//AABBϽ꣬xyzΪֵ
};

//Χݽṹ
struct Sphere
{
	D3DXVECTOR3 vCenter;  //꣬ϵ
	float		fRadius;  //İ뾶
};

HWND					g_hWnd = NULL;
LPDIRECT3D9				g_pD3D = NULL;
LPDIRECT3DDEVICE9		g_pDevice = NULL;
LPD3DXMESH				g_pOBB1Mesh = NULL;
LPD3DXMESH				g_pOBB2Mesh = NULL;
LPD3DXMESH				g_pAABBMesh = NULL;
LPD3DXMESH				g_pSphereMesh = NULL;
LPD3DXMESH				g_pCylinderMesh = NULL;
LPD3DXFONT				g_pFont = NULL;
AABB					g_AABB;					//AABBMeshİ
OBB						g_OBB1;					//obbmesh1İ
OBB						g_OBB2;					//obbmesh2İ
Sphere					g_Sphere;				//obbmesh3İ
D3DXPLANE				g_Plane;				//plane
LPDIRECT3DVERTEXBUFFER9	g_pAABBVtx = NULL;		//ʾƵAABBĶ
LPDIRECT3DVERTEXBUFFER9	g_pOBB1Vtx = NULL;		//ʾƵOBB1Ķ
LPDIRECT3DVERTEXBUFFER9	g_pOBB2Vtx = NULL;		//ʾƵAABBĶ
LPDIRECT3DVERTEXBUFFER9 g_pLineVtx = NULL;		//ʾһĶ㻺
LPDIRECT3DVERTEXBUFFER9 g_pPointVtx = NULL;		//ʾһ߶εĶ㻺
LPDIRECT3DVERTEXBUFFER9 g_pPlaneVtx = NULL;		//ʾһƽĶ㻺

float					g_fOBB1Width = 2.0f;
float					g_fOBB1Height = 1.0f;
float					g_fOBB1Depth = 1.0f;
float					g_fOBB2Width = 1.0f;
float					g_fOBB2Height = 1.0f;
float					g_fOBB2Depth = 1.0f;
float					g_fAABBWidth =12.0f;
float					g_fAABBHeight = 1.0f;
float					g_fAABBDepth = 1.0f;
D3DXMATRIX				g_matOBB1Rot;
D3DXMATRIX				g_matOBB2Rot;
float                   g_fOBB1RotAngle = 0.0f;
float					g_fOBB2RotAngle = 0.0f;
float					g_fFOO = 0.0f;
BOOL					g_bOBB1OBB2 = FALSE;
BOOL					g_bOBB1Sphere = FALSE;
BOOL					g_bOBB1Point = FALSE;
BOOL					g_bOBB1Line = FALSE;
BOOL					g_bOBB1Plane = FALSE;
BOOL					g_bOBB1Circle = FALSE;

D3DXVECTOR3				g_vPointPos(0.7f,0.0f,0.6f);
D3DXVECTOR3				g_vLineStart(3.0f,0.0f,0.6f);
D3DXVECTOR3				g_vLineEnd(0.0f,0.0f,1.0f);
D3DXVECTOR3				g_vPlaneVtx[4];


void DrawCylinder();
BOOL Intersection( const D3DXPLANE& plane , const OBB& obb );
BOOL Intersection( const D3DXVECTOR3& vCenter , 
				   const D3DXVECTOR3& vRadiusDir ,
				   const D3DXVECTOR3& vVerticalDir , 
				   const float fRadius , const OBB& obb );
BOOL IsSegmentIntersectOBB( const OBB& obb , const D3DXVECTOR3& vStart , const D3DXVECTOR3& vEnd );
BOOL IsPointInOBB( const D3DXVECTOR3& vPos , const OBB& obb );
BOOL Intersection( const OBB& obb , const D3DXVECTOR3& vPos );
BOOL Intersection( const OBB& OBB1 , const OBB& OBB2 );
BOOL Intersection( const D3DXVECTOR3& vCenter , const float fRadius , const OBB& obb );
BOOL Init();
void Shutdown();
BOOL InitMesh();
void Render();
void ComputeOBBVertex( const OBB& obb , Vertex* pVtx );
void ComputeOBBVertex( const OBB& obb , Vertex* pVtx , int nVtxNum );
BOOL UpdateBoundingBoxVertices();
void InitBoundingBox();
void UpdateOBBData( OBB& obb , const D3DXMATRIX& matTransform );
void DrawBoundingBox();
void DrawText();
void DrawOBB1Mesh();
void DrawOBB2Mesh();
void DrawSphereMesh();

void DrawCylinder()
{
	D3DXMATRIX mat1 , mat2 , mat3;
	D3DXMatrixIdentity( &mat1 );
	D3DXMatrixIdentity( &mat2 );
	D3DXMatrixRotationX( &mat1 , D3DXToRadian(90.0f));
	D3DXMatrixTranslation( &mat2 ,3.0f , 0.0f , 0.0f );
	mat3 = mat2 * mat1;
	g_pDevice->SetTransform( D3DTS_WORLD , &mat3 );
	g_pCylinderMesh->DrawSubset(0);
}
//ĳԲǷǺOBBཻԲƽǷOBBཻȻ
//ཻж
//vCenterԲ꣬vRadiusDirĳ뾶ķ,vOtherRadiusDirһ뾶ķ
//fRadiusǰ뾶ĳ.Բķʹ֣vRadiusDirתvOtherRadiusDirĲ
BOOL Intersection( const D3DXVECTOR3& vCenter , 
				  const D3DXVECTOR3& vRadiusDir ,const D3DXVECTOR3& vOtherRadiusDir , const float fRadius , const OBB& obb )
{
	D3DXPLANE Circle;//Բڵƽ
	
	D3DXVECTOR3 tmp;
	D3DXVec3Cross(&tmp,&vRadiusDir , &vOtherRadiusDir );
	D3DXPlaneFromPointNormal( &Circle , &vCenter , &tmp );
	
	//obbԲڵƽ涼ֱཻӷfalse
	if( !Intersection( Circle , obb ) )
		return FALSE;
	
	//򣬾ͼobb8㵽ƽͶӰԲĵľ룬ֻҪһ
	//ͶӰԲĵľСڰ뾶֤obbԲཻ

	float distance_square = 0.0f; //ͶӰԲĵľƽ
	D3DXVECTOR3 circle_normal(Circle.a , Circle.b , Circle.c );
	float diff = 0.0f;//OBB㵽Բĵľ
	float diff_normal = 0.0f; //OBB㵽ԲĵľԲķϵͶӰ
	for( int i = 0 ; i < 8 ; ++i )
	{
		diff = D3DXVec3Length( &(obb.vVertex[i]-vCenter) );
		diff_normal = D3DXVec3Dot( &(obb.vVertex[i]-vCenter) , &circle_normal);

		distance_square = diff * diff - diff_normal * diff_normal;

		if( fRadius*fRadius > distance_square )
			return TRUE;
	}

	return FALSE;
}

//ĳƽǷOBBཻ
//OBB߽İ볤ƽ淨ϵͶӰ֮OBBĵ㵽ƽľȽ
BOOL Intersection( const D3DXPLANE& plane , const OBB& obb )
{
	D3DXVECTOR3 vPlaneNormal(plane.a , plane.b , plane.c);
	D3DXVECTOR3 vAxis[3];
	vAxis[0] = 0.5f * ( obb.vVertex[7] - obb.vVertex[6] );
	vAxis[1] = 0.5f * ( obb.vVertex[4] - obb.vVertex[6] );
	vAxis[2] = 0.5f * ( obb.vVertex[2] - obb.vVertex[6] );

	//OK,OBBԽƽ淨ߵͶӰİ볤
	float d = fabsf( D3DXVec3Dot( &vAxis[0] , &vPlaneNormal ) )+
			  fabsf( D3DXVec3Dot( &vAxis[1] , &vPlaneNormal ) )+
			  fabsf( D3DXVec3Dot( &vAxis[2] , &vPlaneNormal ) );

	//㵽ƽľ롣
	D3DXVECTOR4 v1 , v2;
	v1.x = obb.vCenter.x ;
	v1.y = obb.vCenter.y ; 
	v1.z = obb.vCenter.z ;
	v1.w = 1.0f;

	v2.x = -plane.a;
	v2.y = -plane.b;
	v2.z = -plane.c;
	v2.w = 0.0f;

	float dot = D3DXPlaneDot(&plane,&v2 );

	if( dot == 0.0f )
		return FALSE;

	//ùʽD = (ƽP  Qλ / (ƽP  ֱߵķ)
	float diff = fabs( D3DXPlaneDot( &plane , &v1 )/dot );
	
	if( diff > d )
		return FALSE;

	return TRUE;
	
}

BOOL IsSegmentIntersectOBB(const OBB& obb , const D3DXVECTOR3& vStart , const D3DXVECTOR3& vEnd )
{
	float fNear = FLT_MIN;
	float fFar = FLT_MAX;
	const float fEpsilon = 0.00001f;

	//߶εĳȺͷʸ
	D3DXVECTOR3 vSegmentDir;
	vSegmentDir = vEnd-vStart;
	float fSegmentLen = D3DXVec3Length( &vSegmentDir );
	D3DXVec3Normalize( &vSegmentDir , &vSegmentDir );

	//OBBе㵽߶ʼ
	D3DXVECTOR3 vDiff = obb.vCenter - vStart;

	//obbXıߣwidthߣķʸ
	D3DXVECTOR3 vAxisDir[3];
	D3DXVec3Normalize( &vAxisDir[0] , &(obb.vVertex[7] - obb.vVertex[6]) );
	D3DXVec3Normalize( &vAxisDir[1] , &(obb.vVertex[4] - obb.vVertex[6]) );
	D3DXVec3Normalize( &vAxisDir[2] , &(obb.vVertex[2] - obb.vVertex[6]) );

	for( int i = 0 ; i < 3 ; ++i )
	{
		float fDot = D3DXVec3Dot( &vAxisDir[i] , &vSegmentDir );
		float r = D3DXVec3Dot( &vAxisDir[i] , &vDiff );

		//obbĺwidthߵƽеĻ
		//if( fabsf(fDot) < fEpsilon )
		//{
		//	/*if( -r - obb.fDims[i] > 0.0f || -r + obb.fDims[i] > 0.0f )
		//		return FALSE;*/
		//}

		float s = D3DXVec3Dot( &vAxisDir[i] , &vSegmentDir );

		//õߺobbĳƽཻ֮㣬Զཻཻ
		float t0 = ( r + obb.fDims[i] )/s;
		float t1 = ( r - obb.fDims[i] )/s;

		if( t0 > t1 )
			swap(t0,t1);

		if( t0 > fNear )
			fNear = t0;
		if( t1 < fFar )
			fFar = t1;

		if( fNear > fFar || fFar < 0.0f )
			return FALSE;
	}

	float fIntersect = 0.0f; //fIntersect߶ʼĽľ
	if( fNear > 0.0f )
		fIntersect = fNear;
	else
		fIntersect = fFar;

	if( fSegmentLen + fEpsilon > fIntersect )//ҪȽ߶ʼĽľʵ߶εĳȵľ
		return TRUE;

	return FALSE;
}

BOOL IsPointInOBB( const D3DXVECTOR3& vPos , const OBB& obb )
{
	//Ҫϸע⹹ƽĲ˳
	D3DXPLANE plane;
	//϶
	D3DXPlaneFromPoints( &plane , &obb.vVertex[2] , &obb.vVertex[0] , &obb.vVertex[3] );
	//n*p+dֵ
	float result = D3DXPlaneDotCoord( &plane , &vPos );
	if( result > 0 )//ʾѾĳ࣬ʾ϶ڸõOBB
		return FALSE;
	else
	{
		//¶
		D3DXPlaneFromPoints( &plane , &obb.vVertex[6] , &obb.vVertex[7] , &obb.vVertex[4] );
		float result = D3DXPlaneDotCoord( &plane , &vPos );
		if( result > 0 )//ʾѾĳ࣬ʾ϶ڸõOBB
			return FALSE;
		else
		{
			//
			D3DXPlaneFromPoints( &plane , &obb.vVertex[7] , &obb.vVertex[6] , &obb.vVertex[3] );
			float result = D3DXPlaneDotCoord( &plane , &vPos );
			if( result > 0 )//ʾѾĳ࣬ʾ϶ڸõOBB
				return FALSE;
			else
			{
				//ǰ
				D3DXPlaneFromPoints( &plane , &obb.vVertex[5] , &obb.vVertex[1] , &obb.vVertex[4] );
				float result = D3DXPlaneDotCoord( &plane , &vPos );
				if( result > 0 )//ʾѾĳ࣬ʾ϶ڸõOBB
					return FALSE;
				else
				{
					//
					D3DXPlaneFromPoints( &plane , &obb.vVertex[2] , &obb.vVertex[6] , &obb.vVertex[0] );
					float result = D3DXPlaneDotCoord( &plane , &vPos );
					if( result > 0 )//ʾѾĳ࣬ʾ϶ڸõOBB
						return FALSE;
					else
					{
						D3DXPlaneFromPoints( &plane , &obb.vVertex[3] , &obb.vVertex[1] , &obb.vVertex[7] );
						float result = D3DXPlaneDotCoord( &plane , &vPos );
						if( result > 0 )//ʾѾĳ࣬ʾ϶ڸõOBB
							return FALSE;
					}
				}
			}
		}
	}

	return TRUE;
}

//
void DrawSphereMesh()
{
	D3DXMATRIX mat;
	D3DXMatrixIdentity( &mat );
	D3DXMatrixTranslation( &mat , 0.0f , 0.0f , 1.5f );
	g_pDevice->SetTransform( D3DTS_WORLD , &mat );
	g_pSphereMesh->DrawSubset(0);
}


//жǷobbཻ
BOOL Intersection( const D3DXVECTOR3& vCenter , const float fRadius , const OBB& obb )
{
	//һȻȡOBBе㵽е
	D3DXVECTOR3 vDiff = vCenter - obb.vCenter;
	float fS[3]    = { 0.0f, 0.0f, 0.0f };
	float fD       = 0.0f;
	
	//2,õobbᣬҹ
	D3DXVECTOR3 OBBAxisDir[3];
	OBBAxisDir[0] = obb.vVertex[7] - obb.vVertex[6];
	OBBAxisDir[1] = obb.vVertex[4] - obb.vVertex[6];
	OBBAxisDir[2] = obb.vVertex[2] - obb.vVertex[6];
	
	for( int i = 0 ; i < 3 ; ++i )
		D3DXVec3Normalize( &OBBAxisDir[i] , &OBBAxisDir[i] );

	const D3DXVECTOR3& OBBCenter    = obb.vCenter;
	const D3DXVECTOR3& SphereCenter = vCenter;

	D3DXVECTOR3 tmp;
	//ߺobbһĵСʾĵxҪСڰΧmin_x˵ķ
	if( D3DXVec3Dot( &vDiff , &OBBAxisDir[0] ) < 0.0f )//Ҳ˵ͶӰOBBAxisDir.[0]
	{
		//OBBCenter - OBBAxisDir.[0]*rkOBBDi.x obbXϵmin_x˵
		//ĵ㣭obbXϵmin_x˵㣩* OBBAxisDir.[0]ʾ
		// ĵobbϵmin_x˵OBBAxisDir.[0]ͶӰ
		tmp = SphereCenter - ( OBBCenter - ( OBBAxisDir[0] * obb.fDims[0] ) );
		fS[0] = D3DXVec3Dot( &tmp , &OBBAxisDir[0] ); 

		if( fS[0] <= -fRadius ) //СڵĻȻǲཻ
			return FALSE;
		else if( fS[0] < 0.0f ) //ͼXķ  
			fD += fS[0] * fS[0];

		fS[0] += fRadius;
	}
	else//ʾĵxҪڰΧmax_x˵ķ
	{
		//ͬϣĵobbϵmax_x˵OBBAxisDir.[0]ͶӰ
		tmp = OBBCenter + OBBAxisDir[0] * obb.fDims[0]  - SphereCenter;
		fS[0] = D3DXVec3Dot( &tmp , &OBBAxisDir[0] );

		if( fS[0] <= -fRadius ) //СڵĻǲཻ
			return FALSE;
		else if( fS[0] < 0.0f )
			fD += fS[0] * fS[0];

		fS[0] += fRadius;
	}

	if( D3DXVec3Dot( &vDiff , &OBBAxisDir[1] ) < 0.0f )
	{
		tmp = SphereCenter - ( OBBCenter - OBBAxisDir[1] * obb.fDims[1] ); 
		fS[1] = D3DXVec3Dot( &tmp , &OBBAxisDir[1] );

		if( fS[1] <= -fRadius )
			return FALSE;
		else if( fS[1] < 0.0f )
			fD += fS[1] * fS[1];

		fS[1] += fRadius;
	}
	else
	{
		tmp = OBBCenter + OBBAxisDir[1] * obb.fDims[1] - SphereCenter;
		fS[1] = D3DXVec3Dot( &tmp , &OBBAxisDir[1]);

		if( fS[1] <= -fRadius )
			return FALSE;
		else if( fS[1] < 0.0f )
			fD += fS[1] * fS[1];

		fS[1] += fRadius;
	}

	if( D3DXVec3Dot( &vDiff , &OBBAxisDir[2] ) < 0.0f )
	{
		tmp = SphereCenter - ( OBBCenter - OBBAxisDir[2] * obb.fDims[2] );
		fS[2] = D3DXVec3Dot( &tmp , &OBBAxisDir[2] );

		if( fS[2] <= -fRadius )
			return FALSE;
		else if( fS[2] < 0.0f )
			fD += fS[2] * fS[2];

		fS[2] += fRadius;
	}
	else
	{
		tmp = OBBCenter + OBBAxisDir[2] * obb.fDims[2] - SphereCenter; 
		fS[2] = D3DXVec3Dot( &tmp,&OBBAxisDir[2]);

		if( fS[2] <= -fRadius )
			return FALSE;
		else if( fS[2] < 0.0f )
			fD += fS[2] * fS[2];

		fS[2] += fRadius;

	}

	
	if( fD <= ( fRadius * fRadius ) )
		return TRUE;

	return FALSE;
}
//жOBBǷཻ
BOOL Intersection( const OBB& OBB1 , const OBB& OBB2 )
{
	const float fParallellCutoff = 0.99999f;
	BOOL bParallellAxes = FALSE;

	//һOBBǵ3ķ
	//ֲOBB1AxisDirOBB2AxisDir洢˳Ϊx,y,z
	D3DXVECTOR3 OBB1AxisDir[3],OBB2AxisDir[3];
	//OBBX᷽ʸעOBBĶ˳򣺴ϵ£ﵽ⣬
	OBB1AxisDir[0] = OBB1.vVertex[7] - OBB1.vVertex[6];
	OBB1AxisDir[1] = OBB1.vVertex[4] - OBB1.vVertex[6];
	OBB1AxisDir[2] = OBB1.vVertex[2] - OBB1.vVertex[6];

	OBB2AxisDir[0] = OBB2.vVertex[7] - OBB2.vVertex[6];
	OBB2AxisDir[1] = OBB2.vVertex[4] - OBB2.vVertex[6];
	OBB2AxisDir[2] = OBB2.vVertex[2] - OBB2.vVertex[6];

	//ʸnormalize
	for( int i = 0 ; i < 3 ; ++i )
	{
		D3DXVec3Normalize( &OBB1AxisDir[i] , &OBB1AxisDir[i] );
		D3DXVec3Normalize( &OBB2AxisDir[i] , &OBB2AxisDir[i] );
	}

	//ڶOBBеߣOBB1ָOBB2,Լ֮ĵ
	D3DXVECTOR3 vCenterDiff = OBB2.vCenter - OBB1.vCenter;

	float afAxisDot[3][3];    // afAxisDot[i][j] = OBB1AxisDir[i] * OBB2AxisDir[j];
	float afAbsAxisDot[3][3]; // afAbsAxisDot[i][j] = |afAxisDot[i][j]|
	float afAxesADotDiff[3];  // OBB1AxisDir[i]  = OBBAxisDir.A[i] * vDiff

	float fProj;

	/*ݷᶨ,ֱOBBΪ*/
	float fOverlap;
	//1 жOBB1XΪ
	{
		afAxisDot[0][0] = D3DXVec3Dot( &OBB1AxisDir[0] , &OBB2AxisDir[0] );
		afAxisDot[0][1] = D3DXVec3Dot( &OBB1AxisDir[0] , &OBB2AxisDir[1] );
		afAxisDot[0][2] = D3DXVec3Dot( &OBB1AxisDir[0] , &OBB2AxisDir[2] );
		//ȡľֵľֵ>0.9999f,Ҳ˵cosֵӽ1ͱʾ
		//֮ʵƽеģǷǳƽУ
		afAbsAxisDot[0][0] = fabsf( afAxisDot[0][0] );  
		if( afAbsAxisDot[0][0] > fParallellCutoff ) 
			bParallellAxes = TRUE;
		afAbsAxisDot[0][1] = fabsf( afAxisDot[0][1] ); 
		if( afAbsAxisDot[0][1] > fParallellCutoff ) 
			bParallellAxes = TRUE;
		afAbsAxisDot[0][2] = fabsf( afAxisDot[0][2] ); 
		if( afAbsAxisDot[0][2] > fParallellCutoff ) 
			bParallellAxes = TRUE;

		//1X12ӵеߵĵĿ
		//Ϊ˼е1еһϵͶӰĳ
		afAxesADotDiff[0]  = D3DXVec3Dot( &OBB1AxisDir[0] , &vCenterDiff );
		//е1XϵͶӰǷ2ӵİ2ӵİ
		//ϵͶӰ֮ͣڵĻôͱʾOBBġ
		fOverlap = fabsf( afAxesADotDiff[0] ) - ( OBB1.fDims[0] + 
			OBB2.fDims[0] * afAbsAxisDot[0][0] + 
			OBB2.fDims[1] * afAbsAxisDot[0][1] + 
			OBB2.fDims[2] * afAbsAxisDot[0][2]);
		if( fOverlap > 0.0f )
			return false;
	}

	//2 жOBB1ZΪXΪһ
	{
		afAxisDot[1][0] = D3DXVec3Dot( &OBB1AxisDir[1] , &OBB2AxisDir[0] );
		afAxisDot[1][1] = D3DXVec3Dot( &OBB1AxisDir[1] , &OBB2AxisDir[1] );
		afAxisDot[1][2] = D3DXVec3Dot( &OBB1AxisDir[1] , &OBB2AxisDir[2] );
		//ȡľֵľֵ>0.9999f,Ҳ˵cosֵӽ1ͱʾ
		//֮ʵƽеģǷǳƽУ
		afAbsAxisDot[1][0] = fabsf( afAxisDot[1][0] );  
		if( afAbsAxisDot[1][0] > fParallellCutoff ) 
			bParallellAxes = TRUE;

		afAbsAxisDot[1][1] = fabsf( afAxisDot[1][1] ); 
		if( afAbsAxisDot[1][1] > fParallellCutoff ) 
			bParallellAxes = TRUE;

		afAbsAxisDot[1][2] = fabsf( afAxisDot[1][2] ); 
		if( afAbsAxisDot[1][2] > fParallellCutoff ) 
			bParallellAxes = TRUE;


		afAxesADotDiff[1] = D3DXVec3Dot( &OBB1AxisDir[1] , &vCenterDiff );

		fOverlap = fabsf( afAxesADotDiff[1] ) - ( OBB1.fDims[1] + 
			OBB2.fDims[0] * afAbsAxisDot[1][0] + 
			OBB2.fDims[1] * afAbsAxisDot[1][1] + 
			OBB2.fDims[2] * afAbsAxisDot[1][2] );
		if( fOverlap > 0.0f )
			return FALSE;
	}
	//3 жOBB1YΪ
	{
		afAxisDot[2][0] = D3DXVec3Dot( &OBB1AxisDir[2] , &OBB2AxisDir[0] );
		afAxisDot[2][1] = D3DXVec3Dot( &OBB1AxisDir[2] , &OBB2AxisDir[1] );
		afAxisDot[2][2] = D3DXVec3Dot( &OBB1AxisDir[2] , &OBB2AxisDir[2] );

		afAbsAxisDot[2][0] = fabsf( afAxisDot[2][0] );  
		if( afAbsAxisDot[2][0] > fParallellCutoff ) 
			bParallellAxes = TRUE;

		afAbsAxisDot[2][1] = fabsf( afAxisDot[2][1] ); 
		if( afAbsAxisDot[2][1] > fParallellCutoff ) 
			bParallellAxes = TRUE;

		afAbsAxisDot[2][2] = fabsf( afAxisDot[2][2] ); 
		if( afAbsAxisDot[2][2] > fParallellCutoff ) 
			bParallellAxes = TRUE;

		afAxesADotDiff[2] = D3DXVec3Dot( &OBB1AxisDir[2] , &vCenterDiff );

		fOverlap = fabsf( afAxesADotDiff[2] ) - ( OBB1.fDims[2] + 
			OBB2.fDims[0] * afAbsAxisDot[2][0] + 
			OBB2.fDims[1] * afAbsAxisDot[2][1] + 
			OBB2.fDims[2] * afAbsAxisDot[2][2]);
		if( fOverlap > 0.0f )
			return FALSE;
	}

	//4 ж0BB2XΪ,OB1ԭһ
	{
		//ΪǰѾ֮ĵˡ
		//ڿֱӼеBXͶӰ
		fProj = D3DXVec3Dot( &OBB2AxisDir[0] , &vCenterDiff );
		fOverlap = fabsf( fProj ) - ( OBB2.fDims[0] + 
			OBB1.fDims[0] * afAbsAxisDot[0][0] + 
			OBB1.fDims[1] * afAbsAxisDot[1][0] + 
			OBB1.fDims[2] * afAbsAxisDot[2][0]);
		if( fOverlap > 0.0f )
			return FALSE;
	}
	//5 ж0BB2zΪ
	{
		fProj = D3DXVec3Dot( &OBB2AxisDir[1] , &vCenterDiff );
		fOverlap = fabsf( fProj ) - ( OBB2.fDims[1] + 
			OBB1.fDims[0] * afAbsAxisDot[0][1] + 
			OBB1.fDims[1] * afAbsAxisDot[1][1] + 
			OBB1.fDims[2] * afAbsAxisDot[2][1]);
		if( fOverlap > 0.0f )
			return FALSE;

	}
	//6 ж0BB2YΪ
	{
		fProj = D3DXVec3Dot( &OBB2AxisDir[2] , &vCenterDiff );
		fOverlap = fabsf( fProj ) - ( OBB2.fDims[2] + 
			OBB1.fDims[0] * afAbsAxisDot[0][2] + 
			OBB1.fDims[1] * afAbsAxisDot[1][2] + 
			OBB1.fDims[2] * afAbsAxisDot[2][2]);
		if( fOverlap > 0.0f )
			return FALSE;
	}
	//OBBƽеĻ֮ĲΪ
	if( !bParallellAxes )
	{
		//7 жOBB1XOBB2XĲΪ
		{
			fProj = afAxisDot[1][0] * afAxesADotDiff[2] - afAxisDot[2][0] * afAxesADotDiff[1];
			fOverlap = fabsf(fProj) - ( ( OBB1.fDims[1] * afAbsAxisDot[2][0] + OBB1.fDims[2] * afAbsAxisDot[1][0] ) + 
				( OBB2.fDims[1] * afAbsAxisDot[0][2] + OBB2.fDims[2] * afAbsAxisDot[0][1] ) );
			if( fOverlap > 0.0f )
				return false;
		}
		//8 жOBB1XOBB2YĲΪ
		{
			fProj = afAxisDot[1][1] * afAxesADotDiff[2] - afAxisDot[2][1] * afAxesADotDiff[1];
			fOverlap = fabsf(fProj) - ( (OBB1.fDims[1] * afAbsAxisDot[2][1] + OBB1.fDims[2] * afAbsAxisDot[1][1] ) + 
				(OBB2.fDims[0] * afAbsAxisDot[0][2] + OBB2.fDims[2] * afAbsAxisDot[0][0] ) );
			if( fOverlap > 0.0f )
				return false;
		}
		//9 жOBB1XOBB2ZĲΪ
		{
			fProj = afAxisDot[1][2] * afAxesADotDiff[2] - afAxisDot[2][2] * afAxesADotDiff[1];
			fOverlap = fabsf(fProj) - ( ( OBB1.fDims[1] * afAbsAxisDot[2][2] + OBB1.fDims[2] * afAbsAxisDot[1][2] ) + 
				( OBB2.fDims[0] * afAbsAxisDot[0][1] + OBB2.fDims[1] * afAbsAxisDot[0][0] ) );
			if( fOverlap > 0.0f )
				return false;
		}
		//10 жOBB1YOBB2XĲΪ
		{
			fProj = afAxisDot[2][0] * afAxesADotDiff[0] - afAxisDot[0][0] * afAxesADotDiff[2];
			fOverlap = fabsf( fProj ) - ( ( OBB1.fDims[0] * afAbsAxisDot[2][0] + OBB1.fDims[2] * afAbsAxisDot[0][0] ) + 
				( OBB2.fDims[1] * afAbsAxisDot[1][2] + OBB2.fDims[2] * afAbsAxisDot[1][1] ) );
			if( fOverlap > 0.0f )
				return false;
		}
		//11 жOBB2YOBB2YĲΪ
		{
			fProj = afAxisDot[2][1] * afAxesADotDiff[0] - afAxisDot[0][1] * afAxesADotDiff[2];
			fOverlap = fabsf(fProj) - ( (OBB1.fDims[0] * afAbsAxisDot[2][1] + OBB1.fDims[2] * afAbsAxisDot[0][1] ) + 
				(OBB2.fDims[0] * afAbsAxisDot[1][2] + OBB2.fDims[2] * afAbsAxisDot[1][0] ) );
			if( fOverlap > 0.0f )
				return false;
		}
		//12 жOBB3YOBB2ZĲΪ
		{
			fProj = afAxisDot[2][2] * afAxesADotDiff[0] - afAxisDot[0][2] * afAxesADotDiff[2];
			fOverlap = fabsf( fProj ) - ( ( OBB1.fDims[0] * afAbsAxisDot[2][2] + OBB1.fDims[2] * afAbsAxisDot[0][2] ) + 
				( OBB2.fDims[0] * afAbsAxisDot[1][1] + OBB2.fDims[1] * afAbsAxisDot[1][0] ) );
			if( fOverlap > 0.0f )
				return false;
		}
		//13 жOBB1ZOBB2XĲΪ
		{
			fProj = afAxisDot[0][0] * afAxesADotDiff[1] - afAxisDot[1][0] * afAxesADotDiff[0];
			fOverlap = fabsf(fProj) - ( ( OBB1.fDims[0] * afAbsAxisDot[1][0] + OBB1.fDims[1] * afAbsAxisDot[0][0] ) +
				( OBB2.fDims[1] * afAbsAxisDot[2][2] + OBB2.fDims[2] * afAbsAxisDot[2][1] ) );
			if( fOverlap > 0.0f )
				return false;
		}
		//14 жOBB2ZOBB2YĲΪ
		{
			fProj = afAxisDot[0][1] * afAxesADotDiff[1] - afAxisDot[1][1] * afAxesADotDiff[0];
			fOverlap = fabsf(fProj) - ( ( OBB1.fDims[0] * afAbsAxisDot[1][1] + OBB1.fDims[1] * afAbsAxisDot[0][1] ) + 
				( OBB2.fDims[0] * afAbsAxisDot[2][2] + OBB2.fDims[2] * afAbsAxisDot[2][0] ) );
			if( fOverlap > 0.0f )
				return false;
		}
		//15 жOBB3ZOBB2ZĲΪ
		{
			fProj = afAxisDot[0][2] * afAxesADotDiff[1] - afAxisDot[1][2] * afAxesADotDiff[0];
			fOverlap = fabsf(fProj) - ( ( OBB1.fDims[0] * afAbsAxisDot[1][2] + OBB1.fDims[1] * afAbsAxisDot[0][2] ) + 
				( OBB2.fDims[0] * afAbsAxisDot[2][1] + OBB2.fDims[1] * afAbsAxisDot[2][0] ) );
			if( fOverlap > 0.0f )
				return false;
		}
	}

	return TRUE;
}


//ư
void DrawBoundingBox()
{
	D3DXMATRIX	matIdentity;
	D3DXMatrixIdentity( &matIdentity );
	g_pDevice->SetTransform( D3DTS_WORLD , &matIdentity );
	//obb1İ
	g_pDevice->SetFVF( D3DFVF_MY_VERTEX );
	g_pDevice->SetStreamSource( 0 , g_pOBB1Vtx , 0 , sizeof(Vertex) );
	g_pDevice->DrawPrimitive( D3DPT_LINELIST , 0 , 12 );

	//obb2İ
	g_pDevice->SetFVF( D3DFVF_MY_VERTEX );
	g_pDevice->SetStreamSource( 0 , g_pOBB2Vtx , 0 , sizeof(Vertex) );
	g_pDevice->DrawPrimitive( D3DPT_LINELIST , 0 , 12 );

}

//°еϢ,ע⣬ֱӰobb꣬ȵȶиı
void UpdateOBBData( OBB& obb , const D3DXMATRIX& matTransform )
{
	D3DXVECTOR4 out;
	for( int i = 0 ; i < 8 ; ++i )
	{
		D3DXVec3Transform( &out , &obb.vVertex[i] , &matTransform );
		obb.vVertex[i].x = out.x;
		obb.vVertex[i].y = out.y;
		obb.vVertex[i].z = out.z;
	}
	//е
	D3DXVec3Transform( &out , &obb.vCenter , &matTransform );
	obb.vCenter.x = out.x;
	obb.vCenter.y = out.y;
	obb.vCenter.z = out.z;	
	//°볤
	obb.fDims[0] = 0.5f * D3DXVec3Length( &(obb.vVertex[7]-obb.vVertex[6]) );
	obb.fDims[1] = 0.5f * D3DXVec3Length( &(obb.vVertex[4]-obb.vVertex[6]) );
	obb.fDims[2] = 0.5f * D3DXVec3Length( &(obb.vVertex[2]-obb.vVertex[6]) );
}
//ʼаϢ
void InitBoundingBox()
{
	//ʼobb1
	g_OBB1.fDims[0] = g_fOBB1Width * 0.5f;
	g_OBB1.fDims[1] = g_fOBB1Depth * 0.5f;
	g_OBB1.fDims[2] = g_fOBB1Height * 0.5f;
	g_OBB1.vCenter = D3DXVECTOR3(0.0f,0.0f,0.0f);
	//ʼ8
	g_OBB1.vVertex[0] = D3DXVECTOR3(-1.0f,0.5f,0.5f);
	g_OBB1.vVertex[1] = D3DXVECTOR3(1.0f,0.5f,0.5f);
	g_OBB1.vVertex[2] =	D3DXVECTOR3(-1.0f,0.5f,-0.5f);
	g_OBB1.vVertex[3] = D3DXVECTOR3(1.0f,0.5f,-0.5f);
	g_OBB1.vVertex[4] = D3DXVECTOR3(-1.0f,-0.5f,0.5f);
	g_OBB1.vVertex[5] = D3DXVECTOR3(1.0f,-0.5f,0.5f);
	g_OBB1.vVertex[6] =	D3DXVECTOR3(-1.0f,-0.5f,-0.5f);
	g_OBB1.vVertex[7] = D3DXVECTOR3(1.0f,-0.5f,-0.5f);
	
	//ʼobb2
	g_OBB2.fDims[0] = g_fOBB2Width * 0.5f;
	g_OBB2.fDims[1] = g_fOBB2Depth * 0.5f;
	g_OBB2.fDims[2] = g_fOBB2Height * 0.5f;
	g_OBB2.vCenter = D3DXVECTOR3(0.0f,0.0f,0.0f);
	
	//ʼ8
	g_OBB2.vVertex[0] = D3DXVECTOR3(-0.5f,0.5f,0.5f);
	g_OBB2.vVertex[1] = D3DXVECTOR3(0.5f,0.5f,0.5f);
	g_OBB2.vVertex[2] =	D3DXVECTOR3(-0.5f,0.5f,-0.5f);
	g_OBB2.vVertex[3] = D3DXVECTOR3(0.5f,0.5f,-0.5f);
	g_OBB2.vVertex[4] = D3DXVECTOR3(-0.5f,-0.5f,0.5f);
	g_OBB2.vVertex[5] = D3DXVECTOR3(0.5f,-0.5f,0.5f);
	g_OBB2.vVertex[6] =	D3DXVECTOR3(-0.5f,-0.5f,-0.5f);
	g_OBB2.vVertex[7] = D3DXVECTOR3(0.5f,-0.5f,-0.5f);
	
	//obbƽƵ(1.0f,0.0f,1.0f)
	D3DXMATRIX matIdentity;
	D3DXMatrixIdentity( &matIdentity );
	D3DXMatrixTranslation( &matIdentity ,0.0f , 0.0f , -1.5f );
	UpdateOBBData( g_OBB2 , matIdentity );

	//ʼaabb
	g_AABB.vMin = D3DXVECTOR3( -g_fOBB2Width*0.5f , -g_fOBB2Height*0.5f , -g_fOBB2Height*0.5f);
	g_AABB.vMax = D3DXVECTOR3( g_fOBB2Width*0.5f , g_fOBB2Height*0.5f , g_fOBB2Height*0.5f);

	//ʼðеش
	UpdateBoundingBoxVertices();
}



BOOL Init()
{
	g_pD3D = Direct3DCreate9( D3D_SDK_VERSION );

	D3DDISPLAYMODE d3ddm;

	g_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm );

	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory( &d3dpp, sizeof(d3dpp) );

	d3dpp.Windowed               = TRUE;
	d3dpp.SwapEffect             = D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat       = d3ddm.Format;
	d3dpp.EnableAutoDepthStencil = TRUE;
	d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
	d3dpp.PresentationInterval   = D3DPRESENT_INTERVAL_IMMEDIATE;

	g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, g_hWnd,
		D3DCREATE_SOFTWARE_VERTEXPROCESSING,
		&d3dpp, &g_pDevice );

	//Ⱦ״̬
	g_pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	g_pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	g_pDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);

	//worldviewprojection
	D3DXMATRIX matTransform;
	D3DXMatrixPerspectiveFovLH( &matTransform, D3DXToRadian( 45.0f ), 640.0f / 480.0f, 1.0f, 100.0f );
	g_pDevice->SetTransform( D3DTS_PROJECTION, &matTransform );
	D3DXMatrixLookAtLH( &matTransform , &D3DXVECTOR3(0.0f,15.0f,0.0f),
						&D3DXVECTOR3(0.0f,0.0f,0.0f),&D3DXVECTOR3(0.0f,0.0f,1.0f));
	g_pDevice->SetTransform( D3DTS_VIEW , &matTransform );
	D3DXMatrixIdentity( &matTransform );
	g_pDevice->SetTransform( D3DTS_WORLD , &matTransform );
	
	//ʼ߶ζ͵꣬
	
	BYTE* pVertices = NULL;
	Vertex linevtx[2];
	Vertex pointvtx;
	Vertex planevtx[8];

	linevtx[0].diffuse = 0xFF0000FF;
	linevtx[0].x = g_vLineStart.x; 
	linevtx[0].y = g_vLineStart.y;
	linevtx[0].z = g_vLineStart.z;
	linevtx[1].diffuse = 0xFF0000FF;
	linevtx[1].x = g_vLineEnd.x; 
	linevtx[1].y = g_vLineEnd.y; 
	linevtx[1].z = g_vLineEnd.z;
	
	
	pointvtx.diffuse = 0xFFFF0000;
	pointvtx.x = g_vPointPos.x;
	pointvtx.y = g_vPointPos.y ;
	pointvtx.z = g_vPointPos.z;

	//ƽĸ
	g_vPlaneVtx[0] = D3DXVECTOR3(-0.5f,-0.5f,1.9f);
	g_vPlaneVtx[1] = D3DXVECTOR3(0.5f,-0.5f,1.9f);
	g_vPlaneVtx[2] = D3DXVECTOR3(-0.5f,0.5f,0.9f);
	g_vPlaneVtx[3] = D3DXVECTOR3(0.5f,0.5f,0.9f);

	//ʼƽ
	D3DXPlaneFromPoints( &g_Plane , &g_vPlaneVtx[2],
						 &g_vPlaneVtx[0] , &g_vPlaneVtx[3] );

	//ʼƽĸ
	const int index[8] = {0,1,1,3,3,2,2,0};
	
	for( int i = 0 ; i < 8 ; ++i )
	{
		planevtx[i].diffuse = 0xFFFF00FF;
		planevtx[i].x = g_vPlaneVtx[index[i]].x;
		planevtx[i].y = g_vPlaneVtx[index[i]].y;
		planevtx[i].z = g_vPlaneVtx[index[i]].z;
	}

	HRESULT hr = g_pDevice->CreateVertexBuffer( 2*sizeof(Vertex), 0, D3DFVF_MY_VERTEX,
		D3DPOOL_DEFAULT, &g_pLineVtx,NULL );

	hr = g_pLineVtx->Lock( 0, 2*sizeof(Vertex), (void**)&pVertices, 0 );
	memcpy( pVertices, linevtx, sizeof(linevtx) );
	g_pLineVtx->Unlock();

	g_pDevice->CreateVertexBuffer( sizeof(Vertex), 0, D3DFVF_MY_VERTEX,
		D3DPOOL_DEFAULT, &g_pPointVtx,NULL );

	hr = g_pPointVtx->Lock( 0, sizeof(Vertex), (void**)&pVertices, 0 );
	memcpy( pVertices, &pointvtx, sizeof(pointvtx) );
	g_pPointVtx->Unlock();

	g_pDevice->CreateVertexBuffer( 8*sizeof(Vertex), 0, D3DFVF_MY_VERTEX,
		D3DPOOL_DEFAULT, &g_pPlaneVtx,NULL );

	hr = g_pPlaneVtx->Lock( 0, 8*sizeof(Vertex), (void**)&pVertices, 0 );
	memcpy( pVertices, &planevtx, sizeof(planevtx) );
	g_pPlaneVtx->Unlock();




	//ʼmesh
	InitMesh();
	InitBoundingBox();
	UpdateOBBData( g_OBB1 , matTransform );


	D3DXMatrixIdentity( &g_matOBB1Rot );
	D3DXMatrixIdentity( &g_matOBB2Rot );

	HDC hDC;
	int nHeight;
	int nPointSize = 18;
	hDC = GetDC( NULL );
	nHeight = -( MulDiv( nPointSize, GetDeviceCaps(hDC, LOGPIXELSY), 72 ) );
	ReleaseDC( NULL, hDC );

	hr = D3DXCreateFont( g_pDevice, nHeight*0.5f, 0, FW_BOLD, 0, FALSE, 
		DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, 
		DEFAULT_PITCH | FF_DONTCARE, TEXT("Arial"), 
		&g_pFont );

	return TRUE;
}

//ͷеD3D
void Shutdown()
{
	if( g_pCylinderMesh )
		g_pCylinderMesh->Release();

	if( g_pPlaneVtx )
		g_pPlaneVtx->Release();

	if( g_pPointVtx )
		g_pPointVtx->Release();

	if( g_pLineVtx )
		g_pLineVtx->Release();

	if( g_pFont )
		g_pFont->Release();

	if( g_pSphereMesh )
		g_pSphereMesh->Release();
	
	if( g_pAABBMesh )
		g_pAABBMesh->Release();

	if( g_pOBB1Mesh )
		g_pOBB1Mesh->Release();

	if( g_pOBB2Mesh )
		g_pOBB2Mesh->Release();

	if( g_pAABBVtx )
		g_pAABBVtx->Release();

	if( g_pOBB1Vtx )
		g_pOBB1Vtx->Release();

	if( g_pOBB2Vtx )
		g_pOBB2Vtx->Release();

	if( g_pDevice )
		g_pDevice->Release();

	if( g_pD3D )
		g_pD3D->Release();
}
//ʼеmesh
BOOL InitMesh()
{
	LPD3DXMESH pTmpMesh = NULL;
	LPDIRECT3DVERTEXBUFFER9 pTempVertexBuffer = NULL;
	Vertex *pVertices = NULL;
	int nNumVerts = 0;
	
	//OBB1
	D3DXCreateBox(g_pDevice, g_fOBB1Width, g_fOBB1Height, g_fOBB1Depth , &pTmpMesh, NULL);
	pTmpMesh->CloneMeshFVF( 0, D3DFVF_MY_VERTEX, g_pDevice, &g_pOBB1Mesh );
	if( SUCCEEDED( g_pOBB1Mesh->GetVertexBuffer( &pTempVertexBuffer ) ) )
	{
		nNumVerts = g_pOBB1Mesh->GetNumVertices();
		pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices, 0 );

		for( int i = 0; i < nNumVerts; ++i )
				pVertices[i].diffuse = D3DCOLOR_COLORVALUE( 0.0, 0.0, 1.0, 1.0 );

		pTempVertexBuffer->Unlock();
		pTempVertexBuffer->Release();
		pTmpMesh->Release();
	}
	else
		return FALSE;

	//OBB2
	D3DXCreateBox(g_pDevice, g_fOBB2Width, g_fOBB2Height, g_fOBB2Depth , &pTmpMesh, NULL);
	pTmpMesh->CloneMeshFVF( 0, D3DFVF_MY_VERTEX, g_pDevice, &g_pOBB2Mesh );
	if( SUCCEEDED( g_pOBB2Mesh->GetVertexBuffer( &pTempVertexBuffer ) ) )
	{
		nNumVerts = g_pOBB2Mesh->GetNumVertices();

		pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices, 0 );
		{
			for( int i = 0; i < nNumVerts; ++i )
				pVertices[i].diffuse = D3DCOLOR_COLORVALUE( 1.0, 1.0, 0.0, 1.0 );
		}
		pTempVertexBuffer->Unlock();
		pTempVertexBuffer->Release();
		pTmpMesh->Release();
	}
	else
		return FALSE;

	//AABB
	D3DXCreateBox(g_pDevice, g_fAABBWidth, g_fAABBHeight, g_fAABBDepth , &pTmpMesh, NULL);
	pTmpMesh->CloneMeshFVF( 0, D3DFVF_MY_VERTEX, g_pDevice, &g_pAABBMesh );
	if( SUCCEEDED( g_pAABBMesh->GetVertexBuffer( &pTempVertexBuffer ) ) )
	{
		nNumVerts = g_pAABBMesh->GetNumVertices();

		pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices, 0 );
		{
			for( int i = 0; i < nNumVerts; ++i )
				pVertices[i].diffuse = D3DCOLOR_COLORVALUE( 1.0, 1.0, 0.0, 1.0 );
		}
		pTempVertexBuffer->Unlock();
		pTempVertexBuffer->Release();
		pTmpMesh->Release();
	}
	else
		return FALSE;

	//Sphere
	D3DXCreateSphere(g_pDevice, 0.5f, 20, 20 , &pTmpMesh, NULL);
	pTmpMesh->CloneMeshFVF( 0, D3DFVF_MY_VERTEX, g_pDevice, &g_pSphereMesh );
	if( SUCCEEDED( g_pSphereMesh->GetVertexBuffer( &pTempVertexBuffer ) ) )
	{
		nNumVerts = g_pSphereMesh->GetNumVertices();

		pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices, 0 );
		{
			for( int i = 0; i < nNumVerts; ++i )
				pVertices[i].diffuse = D3DCOLOR_COLORVALUE( 1.0, 1.0, 0.0, 1.0 );
		}
		pTempVertexBuffer->Unlock();
		pTempVertexBuffer->Release();
		pTmpMesh->Release();
	}
	else
		return FALSE;

	//cylinder
	D3DXCreateCylinder( g_pDevice , 2.0f , 2.0f , 0.0f,20,20 , &g_pCylinderMesh,NULL );
	pTmpMesh->CloneMeshFVF( 0, D3DFVF_MY_VERTEX, g_pDevice, &g_pCylinderMesh );
	if( SUCCEEDED( g_pCylinderMesh->GetVertexBuffer( &pTempVertexBuffer ) ) )
	{
		nNumVerts = g_pCylinderMesh->GetNumVertices();

		pTempVertexBuffer->Lock( 0, 0, (void**)&pVertices, 0 );
		{
			for( int i = 0; i < nNumVerts; ++i )
				pVertices[i].diffuse = D3DCOLOR_COLORVALUE( 1.0, 0.0, 0.0, 1.0 );
		}
		pTempVertexBuffer->Unlock();
		pTempVertexBuffer->Release();
		pTmpMesh->Release();
	}
	else
		return FALSE;
	
	return TRUE;
}


//ʼ/°ж,עǻõĶ
BOOL UpdateBoundingBoxVertices()
{
	HRESULT		hr;
	Vertex		bounding_box_vtx[24];
	const int	index[24] = {0,1,1,3,3,2,2,0,4,5,5,7,7,6,6,4,0,4,1,5,3,7,2,6};
	
	//24
	if( !g_pOBB1Vtx )
	{
		hr = g_pDevice->CreateVertexBuffer( 24*sizeof(Vertex), 0, D3DFVF_MY_VERTEX,
												D3DPOOL_DEFAULT, &g_pOBB1Vtx,NULL );
		if( FAILED(hr) )
			return FALSE;
	}
	
	//ŰЩֵֵưеʱ㻺ȥ
	for( int i = 0 ; i < 24 ; ++i )
	{
		bounding_box_vtx[i].x = g_OBB1.vVertex[index[i]].x;
		bounding_box_vtx[i].y = g_OBB1.vVertex[index[i]].y;
		bounding_box_vtx[i].z = g_OBB1.vVertex[index[i]].z;
		bounding_box_vtx[i].diffuse = 0xFFFFFFFF;
	}
	//OK,ڿԿĶ㻺ȥ
	BYTE* pVertices = NULL;
	hr = g_pOBB1Vtx->Lock( 0, 24*sizeof(Vertex), (void**)&pVertices, 0 );
	if( FAILED(hr) )
		return FALSE;
	memcpy( pVertices, bounding_box_vtx, sizeof(bounding_box_vtx) );
	g_pOBB1Vtx->Unlock();
	
	
	//24
	if( !g_pOBB2Vtx )
	{
		hr = g_pDevice->CreateVertexBuffer( 24*sizeof(Vertex), 0, D3DFVF_MY_VERTEX,
			D3DPOOL_DEFAULT, &g_pOBB2Vtx,NULL );
		if( FAILED(hr) )
			return FALSE;
	}
	
	//ŰЩֵֵưеʱ㻺ȥ
	for( i = 0 ; i < 24 ; ++i )
	{
		bounding_box_vtx[i].x = g_OBB2.vVertex[index[i]].x;
		bounding_box_vtx[i].y = g_OBB2.vVertex[index[i]].y;
		bounding_box_vtx[i].z = g_OBB2.vVertex[index[i]].z;
		bounding_box_vtx[i].diffuse = 0xFFFFFFFF;
	}
	//OK,ڿԿĶ㻺ȥ
	pVertices = NULL;
	hr = g_pOBB2Vtx->Lock( 0, 24*sizeof(Vertex), (void**)&pVertices, 0 );
	if( FAILED(hr) )
		return FALSE;
	memcpy( pVertices, bounding_box_vtx, sizeof(bounding_box_vtx) );
	g_pOBB2Vtx->Unlock();
	
	//ȡAABBΧеĴڴڴ
	return TRUE;
	
}
//ʾ
void DrawText()
{
	const char* msg1 = "OBB1 intersect with sphere";
	const char* msg2 = "OBB1 intersect with OBB2";
	const char* msg3 = "Single point is in the OBB1";
	const char* msg4 = "Segment intersect with obb1";
	const char* msg5 = "Plane intersect with OBB1";
	const char* msg6 = "circle intersect with obb1";
	RECT destRect;
	SetRect( &destRect, 0, 0, 100, 100 );

	if( g_bOBB1OBB2 )
	{
		g_pFont->DrawText( NULL,msg2, -1, &destRect, DT_NOCLIP, 
		D3DXCOLOR(1.0f, 0.0f, 0.0f, 1.0f) );
	}
	if( g_bOBB1Sphere )
	{
		SetRect( &destRect, 0, 20, 100, 120 );
		g_pFont->DrawText( NULL,msg1, -1, &destRect, DT_NOCLIP, 
			D3DXCOLOR(1.0f, 0.0f, 0.0f, 1.0f) );
	}
	if( g_bOBB1Point )
	{
		SetRect( &destRect, 0, 30, 100, 130 );
		g_pFont->DrawText( NULL,msg3, -1, &destRect, DT_NOCLIP, 
			D3DXCOLOR(1.0f, 0.0f, 0.0f, 1.0f) );
	}
	if( g_bOBB1Line )
	{
		SetRect( &destRect, 0, 40, 100, 140 );
		g_pFont->DrawText( NULL,msg4, -1, &destRect, DT_NOCLIP, 
			D3DXCOLOR(1.0f, 0.0f, 0.0f, 1.0f) );
	}
	if( g_bOBB1Plane )
	{
		SetRect(&destRect, 0, 50, 100, 150 );
		g_pFont->DrawText( NULL,msg5, -1, &destRect, DT_NOCLIP, 
			D3DXCOLOR(1.0f, 0.0f, 1.0f, 1.0f) );
	}
	if( g_bOBB1Circle )
	{
		SetRect(&destRect, 0, 60, 100, 160 );
		g_pFont->DrawText( NULL,msg6, -1, &destRect, DT_NOCLIP, 
			D3DXCOLOR(1.0f, 0.0f, 1.0f, 1.0f) );
	}
}

//obb1meshbounding box
void DrawOBB1Mesh()
{
//	g_pOBB1Mesh->DrawSubset(0);
	g_pDevice->SetFVF( D3DFVF_MY_VERTEX );
	g_pDevice->SetStreamSource( 0 , g_pOBB1Vtx , 0 , sizeof(Vertex) );
	g_pDevice->DrawPrimitive( D3DPT_LINELIST , 0 , 12 );
}

//obb2meshbounding box
void DrawOBB2Mesh()
{
	D3DXMATRIX matIdentity;
	/*D3DXMatrixIdentity( &matIdentity );
	D3DXMatrixTranslation( &matIdentity ,1.5f , 0.0f , 1.0f );
	g_pDevice->SetTransform( D3DTS_WORLD , &matIdentity );
	g_pOBB2Mesh->DrawSubset(0);*/

	D3DXMatrixIdentity( &matIdentity );
	g_pDevice->SetTransform( D3DTS_WORLD , &matIdentity );
	g_pDevice->SetFVF( D3DFVF_MY_VERTEX );
	g_pDevice->SetStreamSource( 0 , g_pOBB2Vtx , 0 , sizeof(Vertex) );
	g_pDevice->DrawPrimitive( D3DPT_LINELIST , 0 , 12 );
}
//Ⱦ
void Render()
{
	
	
	
	//
	D3DXMATRIX matOBB1Rot;
	D3DXMatrixIdentity( &matOBB1Rot );
	D3DXMatrixRotationY( &matOBB1Rot , D3DXToRadian(0.1f) );
	UpdateOBBData( g_OBB1 , matOBB1Rot );
	//»Ƶĵ
	UpdateBoundingBoxVertices();

	g_pDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
							D3DCOLOR_COLORVALUE(0.0f,0.0f,0.0f,1.0f), 1.0f, 0 );
	g_pDevice->BeginScene();
	
	D3DXMatrixIdentity( &matOBB1Rot );
	g_pDevice->SetTransform( D3DTS_WORLD , &matOBB1Rot );

	DrawOBB1Mesh();
//	DrawOBB2Mesh();
//	DrawSphereMesh();
	DrawCylinder();

	//Ƶ
	D3DXMatrixIdentity( &matOBB1Rot );
	g_pDevice->SetTransform( D3DTS_WORLD , &matOBB1Rot );

	g_pDevice->SetFVF( D3DFVF_MY_VERTEX );
	g_pDevice->SetStreamSource( 0 , g_pLineVtx , 0 , sizeof(Vertex) );
	g_pDevice->DrawPrimitive( D3DPT_LINELIST , 0 , 1 );

	g_pDevice->SetFVF( D3DFVF_MY_VERTEX );
	g_pDevice->SetStreamSource( 0 , g_pPointVtx , 0 , sizeof(Vertex) );
	g_pDevice->DrawPrimitive( D3DPT_POINTLIST , 0 , 1 );

	g_pDevice->SetFVF( D3DFVF_MY_VERTEX );
	g_pDevice->SetStreamSource( 0 , g_pPlaneVtx , 0 , sizeof(Vertex) );
	g_pDevice->DrawPrimitive( D3DPT_LINELIST , 0 , 4 );

	//Sleep(100);
	g_bOBB1Point = IsPointInOBB( g_vPointPos , g_OBB1 );
	g_bOBB1Line = IsSegmentIntersectOBB( g_OBB1 , g_vLineStart , g_vLineEnd );
	
	g_bOBB1Plane = Intersection(g_Plane , g_OBB1 );
	g_bOBB1Circle = Intersection( D3DXVECTOR3(3.0f,0.0f,0.0f),
								  D3DXVECTOR3(1.0f,0.0f,0.0f),
								  D3DXVECTOR3(0.0f,0.0f,0.1f),2.0f,g_OBB1 );

	static FLOAT fLastTime = 0.0f;
	static DWORD dwFrames  = 0;
	FLOAT fTime = DXUtil_Timer( TIMER_GETABSOLUTETIME );
	++dwFrames;
	static float m_fFPS = 0.0f;

	
	
	if( fTime - fLastTime > 1.0f )
	{
		m_fFPS    = dwFrames / (fTime - fLastTime);
		fLastTime = fTime;
		dwFrames  = 0;
	}
	RECT destRect;
	SetRect( &destRect , 0 , 0 , 100, 100 );
	const char* fps = "fps = %f";
	char displayfps[32];
	memset(displayfps,NULL,32);
	sprintf( displayfps , fps , m_fFPS );
	g_pFont->DrawText( NULL,displayfps, -1, &destRect, DT_NOCLIP, 
		D3DXCOLOR(1.0f, 0.0f, 0.0f, 1.0f) );

	DrawText();

	g_pDevice->EndScene();
	g_pDevice->Present( NULL, NULL, NULL, NULL );
}
LRESULT CALLBACK WindowProc( HWND   hWnd, UINT   msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{   
	case WM_KEYDOWN:
		{
			switch( wParam )
			{
			case VK_ESCAPE:
				PostQuitMessage(0);
				break;
			case VK_SPACE:
				break;
			case VK_F1:
				break;
			case VK_F2:
				break;
			}
		}
		break;

	case WM_CLOSE:
	case WM_DESTROY:
		PostQuitMessage(0);
		break;

	default:
		return DefWindowProc( hWnd, msg, wParam, lParam );
	}

	return 0;
}

int WINAPI WinMain( HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine, int nCmdShow )
{
	WNDCLASSEX winClass; 
	MSG        uMsg;

	memset(&uMsg,0,sizeof(uMsg));

	winClass.lpszClassName = "MY_WINDOWS_CLASS";
	winClass.cbSize        = sizeof(WNDCLASSEX);
	winClass.style         = CS_HREDRAW | CS_VREDRAW;
	winClass.lpfnWndProc   = WindowProc;
	winClass.hInstance     = hInstance;
	winClass.hIcon         = LoadIcon(hInstance, (LPCTSTR)IDI_APPLICATION);
	winClass.hIconSm       = LoadIcon(hInstance, (LPCTSTR)IDI_APPLICATION);
	winClass.hCursor       = LoadCursor(NULL, IDC_ARROW);
	winClass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	winClass.lpszMenuName  = NULL;
	winClass.cbClsExtra    = 0;
	winClass.cbWndExtra    = 0;

	if( !RegisterClassEx(&winClass) )
		return E_FAIL;

	g_hWnd = CreateWindowEx( NULL, "MY_WINDOWS_CLASS", 
		"Collision Test --- Written by Shinco Xiong",
		WS_OVERLAPPEDWINDOW | WS_VISIBLE,
		0, 0, 640, 480, NULL, NULL, hInstance, NULL );

	if( g_hWnd == NULL )
		return E_FAIL;

	ShowWindow( g_hWnd, nCmdShow );
	UpdateWindow( g_hWnd );

	Init();
	

	while( uMsg.message != WM_QUIT )
	{
		if( PeekMessage( &uMsg, NULL, 0, 0, PM_REMOVE ) )
		{ 
			TranslateMessage( &uMsg );
			DispatchMessage( &uMsg );
		}
		else
		{
			Render();
		}
	}

	UnregisterClass( "MY_WINDOWS_CLASS", winClass.hInstance );
	Shutdown();
	return uMsg.wParam;
}